<?php

namespace Oro\Bundle\TaxBundle\Tests\Unit\Rounding;

use Brick\Math\BigDecimal;
use Brick\Math\RoundingMode;

class RoundingTest extends \PHPUnit_Framework_TestCase
{
    /**
     * @param string $rangeStart
     * @param string $rangeEnd
     * @param string $expected
     *
     * @dataProvider dataProvider
     */
    public function testRound($rangeStart, $rangeEnd, $expected)
    {
        $start = BigDecimal::of($rangeStart);
        $end = BigDecimal::of($rangeEnd);
        $rate = BigDecimal::of('0.05');
        $step = BigDecimal::of('0.01');

        while ($start->isLessThanOrEqualTo($end)) {
            $this->assertEquals($expected, $start->multipliedBy($rate)->toScale(2, RoundingMode::HALF_UP));

            $start = $start->plus($step);
        }
    }

    /**
     * @return array
     *
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    public function dataProvider()
    {
        return [
            ['0.10', '0.29', '0.01'],
            ['0.30', '0.49', '0.02'],
            ['0.50', '0.69', '0.03'],
            ['0.70', '0.89', '0.04'],
            ['0.90', '1.09', '0.05'],
            ['1.10', '1.29', '0.06'],
            ['1.30', '1.49', '0.07'],
            ['1.50', '1.69', '0.08'],
            ['1.70', '1.89', '0.09'],
            ['1.90', '2.09', '0.10'],
            ['2.10', '2.29', '0.11'],
            ['2.30', '2.49', '0.12'],
            ['2.50', '2.69', '0.13'],
            ['2.70', '2.89', '0.14'],
            ['2.90', '3.09', '0.15'],
            ['3.10', '3.29', '0.16'],
            ['3.30', '3.49', '0.17'],
            ['3.50', '3.69', '0.18'],
            ['3.70', '3.89', '0.19'],
            ['3.90', '4.09', '0.20'],
            ['4.10', '4.29', '0.21'],
            ['4.30', '4.49', '0.22'],
            ['4.50', '4.69', '0.23'],
            ['4.70', '4.89', '0.24'],
            ['4.90', '5.09', '0.25'],
            ['5.10', '5.29', '0.26'],
            ['5.30', '5.49', '0.27'],
            ['5.50', '5.69', '0.28'],
            ['5.70', '5.89', '0.29'],
            ['5.90', '6.09', '0.30'],
            ['6.10', '6.29', '0.31'],
            ['6.30', '6.49', '0.32'],
            ['6.50', '6.69', '0.33'],
            ['6.70', '6.89', '0.34'],
            ['6.90', '7.09', '0.35'],
            ['7.10', '7.29', '0.36'],
            ['7.30', '7.49', '0.37'],
            ['7.50', '7.69', '0.38'],
            ['7.70', '7.89', '0.39'],
            ['7.90', '8.09', '0.40'],
            ['8.10', '8.29', '0.41'],
            ['08.30', '08.49', '0.42'],
            ['08.50', '08.69', '0.43'],
            ['08.70', '08.89', '0.44'],
            ['08.90', '09.09', '0.45'],
            ['09.10', '09.29', '0.46'],
            ['09.30', '09.49', '0.47'],
            ['09.50', '09.69', '0.48'],
            ['09.70', '09.89', '0.49'],
            ['09.90', '10.09', '0.50'],
            ['10.10', '10.29', '0.51'],
            ['10.30', '10.49', '0.52'],
            ['10.50', '10.69', '0.53'],
            ['10.70', '10.89', '0.54'],
            ['10.90', '11.09', '0.55'],
            ['11.10', '11.29', '0.56'],
            ['11.30', '11.49', '0.57'],
            ['11.50', '11.69', '0.58'],
            ['11.70', '11.89', '0.59'],
            ['11.90', '12.09', '0.60'],
            ['12.10', '12.29', '0.61'],
            ['12.30', '12.49', '0.62'],
            ['12.50', '12.69', '0.63'],
            ['12.70', '12.89', '0.64'],
            ['12.90', '13.09', '0.65'],
            ['13.10', '13.29', '0.66'],
            ['13.30', '13.49', '0.67'],
            ['13.50', '13.69', '0.68'],
            ['13.70', '13.89', '0.69'],
            ['13.90', '14.09', '0.70'],
            ['14.10', '14.29', '0.71'],
            ['14.30', '14.49', '0.72'],
            ['14.50', '14.69', '0.73'],
            ['14.70', '14.89', '0.74'],
            ['14.90', '15.09', '0.75'],
            ['15.10', '15.29', '0.76'],
            ['15.30', '15.49', '0.77'],
            ['15.50', '15.69', '0.78'],
            ['15.70', '15.89', '0.79'],
            ['15.90', '16.09', '0.80'],
            ['16.10', '16.29', '0.81'],
            ['16.30', '16.49', '0.82'],
            ['16.50', '16.69', '0.83'],
            ['16.70', '16.89', '0.84'],
            ['16.90', '17.09', '0.85'],
            ['17.10', '17.29', '0.86'],
            ['17.30', '17.49', '0.87'],
            ['17.50', '17.69', '0.88'],
            ['17.70', '17.89', '0.89'],
            ['17.90', '18.09', '0.90'],
            ['18.10', '18.29', '0.91'],
            ['18.30', '18.49', '0.92'],
            ['18.50', '18.69', '0.93'],
            ['18.70', '18.89', '0.94'],
            ['18.90', '19.09', '0.95'],
            ['19.10', '19.29', '0.96'],
            ['19.30', '19.49', '0.97'],
            ['19.50', '19.69', '0.98'],
            ['19.70', '19.89', '0.99'],
            ['19.90', '20.09', '1.00'],
            ['20.10', '20.29', '1.01'],
            ['20.30', '20.49', '1.02'],
            ['20.50', '20.69', '1.03'],
            ['20.70', '20.89', '1.04'],
            ['20.90', '21.09', '1.05'],
            ['21.10', '21.29', '1.06'],
            ['21.30', '21.49', '1.07'],
            ['21.50', '21.69', '1.08'],
            ['21.70', '21.89', '1.09'],
            ['21.90', '22.09', '1.10'],
            ['22.10', '22.29', '1.11'],
            ['22.30', '22.49', '1.12'],
            ['22.50', '22.69', '1.13'],
            ['22.70', '22.89', '1.14'],
            ['22.90', '23.09', '1.15'],
            ['23.10', '23.29', '1.16'],
            ['23.30', '23.49', '1.17'],
            ['23.50', '23.69', '1.18'],
            ['23.70', '23.89', '1.19'],
            ['23.90', '24.09', '1.20'],
            ['24.10', '24.29', '1.21'],
            ['24.30', '24.49', '1.22'],
            ['24.50', '24.69', '1.23'],
            ['24.70', '24.89', '1.24'],
            ['24.90', '25.09', '1.25'],
            ['25.10', '25.29', '1.26'],
            ['25.30', '25.49', '1.27'],
            ['25.50', '25.69', '1.28'],
            ['25.70', '25.89', '1.29'],
            ['25.90', '26.09', '1.30'],
            ['26.10', '26.29', '1.31'],
            ['26.30', '26.49', '1.32'],
            ['26.50', '26.69', '1.33'],
            ['26.70', '26.89', '1.34'],
            ['26.90', '27.09', '1.35'],
            ['27.10', '27.29', '1.36'],
            ['27.30', '27.49', '1.37'],
            ['27.50', '27.69', '1.38'],
            ['27.70', '27.89', '1.39'],
            ['27.90', '28.09', '1.40'],
            ['28.10', '28.29', '1.41'],
            ['28.30', '28.49', '1.42'],
            ['28.50', '28.69', '1.43'],
            ['28.70', '28.89', '1.44'],
            ['28.90', '29.09', '1.45'],
            ['29.10', '29.29', '1.46'],
            ['29.30', '29.49', '1.47'],
            ['29.50', '29.69', '1.48'],
            ['29.70', '29.89', '1.49'],
            ['29.90', '30.09', '1.50'],
            ['30.10', '30.29', '1.51'],
            ['30.30', '30.49', '1.52'],
            ['30.50', '30.69', '1.53'],
            ['30.70', '30.89', '1.54'],
            ['30.90', '31.09', '1.55'],
            ['31.10', '31.29', '1.56'],
            ['31.30', '31.49', '1.57'],
            ['31.50', '31.69', '1.58'],
            ['31.70', '31.89', '1.59'],
            ['31.90', '32.09', '1.60'],
        ];
    }
}
